
// (c) COPYRIGHT URI/MIT 1996, 1999
// Please read the full copyright statement in the file COPYRIGHT.
//
// Authors:
//      Dan Holloway <d.holloway@gso.uri.edu> dan

// This is the main CGI program which extracts DDS class attributes from a
// JGOFS data file. In addition, the attributes from an external file (if 
// exist, with the same name as the JGOFS data-object file with suffix '.dds')
// are extracted and added to the in-memory atrribute table before it is 
// sent over the network to the client side. 
//
// DanH 10/21/94

#include "config_jg.h"

static char rcsid[] not_used ={"$Id: jg_dds.cc,v 1.19 2003/01/16 07:57:35 jimg Exp $"};

#include <string>

#include "DDS.h"
#include "DAS.h"
#include "cgi_util.h"
#include "DODSFilter.h"

#include "jgofs.h"
#include "jg_util.h"

const string cgi_version = DODS_SERVER_VERSION;

int 
main(int argc, char * argv[])
{
    DDS dds;
    DAS das;

    DODSFilter df(argc, argv);
	
    if (!df.OK()) {
	df.print_usage();
	return 1;
    }

    if (df.version()) {
        df.set_cgi_version(cgi_version);
	df.send_version_info();
	return 0;
    }

    string object_name = df.get_dataset_name();
    string relative_object_path = "";
    string::size_type slash;
    if ((slash = object_name.find_last_of('/')) != string::npos) {
	relative_object_path = object_name.substr(0,slash+1);
	object_name = object_name.substr(slash+1, 
					 object_name.length()-slash+1);
    }

    try {

	string method_path = "";
	check_jgofs_path(relative_object_path, method_path);

	// Set the filename. 12/22/98 jhrg
	dds.filename(object_name);

	// Determine location of the initial file for the
	// JGOFS data object, using the JGOFS object dictionary.

	char met[256], par[256];
	string pathname = "";
	if (ldctsearch(relative_object_path.c_str(), method_path.c_str(),
		       object_name.c_str(), met, par, 1)) {
	    string objname = par;
	    slash = objname.find_last_of('/');
	    pathname = objname.substr(0, slash+1);
	}
	else {
	    throw Error("jg_dds: could not access local dictionary information");
	}

	df.read_ancillary_das(das, pathname);

	// Open the JGOFS data-object and create a DataSet Descriptor Structure
	create_dds(dds, relative_object_path, method_path, object_name, das);

	df.read_ancillary_dds(dds, pathname);
	df.send_dds(dds, stdout);
    }

    catch (Error &e) {
	ErrMsgT(e.error_message());
	set_mime_text(cout, dods_error, df.get_cgi_version());
	e.print(cout);
	return 1;
    }

    return 0;
}

// $Log: jg_dds.cc,v $
// Revision 1.19  2003/01/16 07:57:35  jimg
// Merged changes from release-3-2-6.
// Removed a number of files used for the client library ans sample clients.
//
// Revision 1.17.4.3  2003/01/16 01:53:02  jimg
// Now uses the new jg_util.cc/h file.
//
// Revision 1.18  2001/08/27 17:41:03  jimg
// Merged with release-3-2-2
//
// Revision 1.17.4.2  2001/08/21 15:33:25  dan
// Extensive modifications to handle exceptions properly, and to clean up
// the object dictionary search logic. Added relative path searching to
// locate the object dictionaries, as per request U.S. JGOFS Office.
//
// Revision 1.17.4.1  2001/06/14 21:15:35  dan
// Added DODS_SERVER_VERSION handling.
//
// Revision 1.17  2000/10/10 23:36:14  jimg
// Added exception processing code.
// Added throws to the read methods.
// Changed the definition of read to match the dap library.
// Moved CVS Log entries to the end of files.
//
// Revision 1.16  2000/01/20 18:58:32  jimg
// Result of merge with release 3-1-2.
//
// Revision 1.15  1999/11/05 01:17:25  jimg
// Result of merge with release-3-1-0
//
// Revision 1.14.2.2  2000/01/19 23:47:15  dan
// Added default path to the .object dictionary as
// the current working directory.  It will look in
// the 'cwd last for this file.
//
// Revision 1.14.2.1  1999/10/24 18:24:15  dan
// Modified to support new DODS_Type and missing_value
// attribute usage.
//
// Revision 1.13.2.1  1999/06/07 17:30:00  dan
// Changed string class usage 'data()' to 'c_str()'.
//
// Revision 1.13  1999/05/27 15:50:38  dan
// Added new logic to find ancillary DDS files relative to the
// location of the top-most file defining a JGOFS data object,
// as defined in the object dictionary.
//
// Revision 1.12  1999/05/25 16:32:41  dan
// Migrated to DODSFilter for command-line processing.
//
// Revision 1.11  1999/05/05 04:38:00  brent
// convert String.h code to new stanard lib <string>
//
// Revision 1.10  1998/03/20 00:30:00  jimg
// Changed calls to the set_mime_*() to match the new parameters
//
// Revision 1.9  1997/06/06 03:54:13  jimg
// Changed call of create_dds() to the new version of that function.
//
// Revision 1.8  1997/03/27 18:29:30  jimg
// Update for version 2.13
//
// Revision 1.7  1997/03/10 23:01:37  dan
// Added DODS Error-handling.
//
// Revision 1.6  1996/11/12 17:01:15  jimg
// Added MIME error returns for certain errors.
//
// Revision 1.5  1996/09/17 00:04:10  jimg
// Misc changes associated with merging branch code to the trunk.
//
// Revision 1.4.6.1.2.1  1996/06/26 22:18:20  jimg
// Dans changes for 2.05.
//
// Revision 1.4  1995/10/11  23:06:26  jimg
// These files are the result of merging Dan's changes into the main
// trunk. However, the archive was messed up in the process, so it may be hard
// to trace back to the original versions of some of these files.
//
// Revision 1.3.2.1  1995/09/18  22:48:04  jimg
// Merged Dan's changes with the trunk version. This will be commited to the
// release-1-0-patches branch and then merged back into the trunk later.
//
// Revision 1.3  1995/07/09  21:40:02  jimg
// Added copyright notice.
//
// Revision 1.2  1995/03/14  02:15:13  jimg
// Modified for the new netio library.
// Removed old code which used C style strings (now it uses C++ Strings).
// Fixed misc bugs.
// Added comments.
//
// Revision 1.1  1995/01/09  16:04:47  dan
// These files constitute the JGOFS server and client library code
// for use with the DODS api.
//
// Revision 1.2  1994/10/06  17:53:06  jimg
// Some reformatting of code for emacs, ...
// Fixed Makefile.in so that two targets are built: client and server.
// Fixed cast away of const in ncdas.cc.
